#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <errno.h>
#include "pcx.h"
#include "editskin.h"
#include "ranges.h"
#include "config.h"

void writerror(char *c, int err)
{
 char s[80];
 
 memset(s,0,80);
 
 switch(err)
 {
  case EPERM:	sprintf(s,"operation not permitted");	break;
  case EINTR:	sprintf(s,"operation interrupted");	break;
  case EIO:	sprintf(s,"I/O error");			break;
  case ENOMEM:	sprintf(s,"out of memory");		break;
  case EACCES:	sprintf(s,"permission denied");		break;
  case EBUSY:	sprintf(s,"device busy");		break;
  case EISDIR:	sprintf(s,"%s is a directory",c);	break;
  case EMFILE:	sprintf(s,"too many open files");	break;
  case ENOSPC:	sprintf(s,"no space left to write %s",c); break;
  case EROFS:	sprintf(s,"read-only file system");	break;
  case ENAMETOOLONG: sprintf(s,"file name %s too long",c); break;
  case EDQUOT:	sprintf(s,"quota exceeded");		break;
 }

 es_write(SAVEBOX_PX+1,SAVEBOX_PY+2,"                                   ");
 es_write(SAVEBOX_PX+1,SAVEBOX_PY+2,s);
 sleep(2);
}

//********************************************* WRITE PCX 
void write_pcx(char *name,
               pcx_header pcx,
               unsigned char *wdata,
               unsigned char *wpalette)
{
 int fd,x,y,z,width;
 long size;
 unsigned char r[1],g[1];
 
 
 fd=creat(name, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH); 

 if (fd==-1) { writerror(name, errno); return; }

 // WRITE HEADER
 if (write(fd,&pcx,sizeof(pcx_header))!=sizeof(pcx_header))
  { writerror(name,errno); close(fd); return; }

 // WRITE DATA
 size=(long)(pcx.xmax-pcx.xmin+1)*(long)(pcx.ymax-pcx.ymin+1);
 x=0; z=0; width=pcx.xmax-pcx.xmin+1;
 
 while (x<size)
 {
  r[0]=wdata[x];
  y=1;
  while (wdata[x]==wdata[x+y] && y<63 && y+x<size && y+z<width) y++;

  if (y>1 || wdata[x]>191)
  { 
   g[0]=192+y;
   if (write(fd,g,1)!=1)
    { writerror(name,errno); close(fd); return; }
   if (write(fd,r,1)!=1)
    { writerror(name,errno); close(fd); return; }
   x+=y;
   z+=y; if (z==width) z=0;
  } else
  {
   if (write(fd,r,1)!=1)
    { writerror(name,errno); close(fd); return; }
   x++;
   z++; if (z==width) z=0;
  }
 }
 r[0]=12;
 if (write(fd,r,1)!=1)
  { writerror(name,errno); close(fd); return; }

 // WRITE PALETTE
 if (write(fd,wpalette,256*3)!=256*3)
  { writerror(name,errno); close(fd); return; }

 close(fd);
}


pcx_header read_pcx(char *name)
{
 int fd,x,y,width,height;
 long size;
 unsigned char r[1],g[1];
 pcx_header wpcx;
 
 palette=(char *) malloc(256*3);
 
 fd=open(name,O_RDONLY);

 if (fd==-1) { perror(name); exit(1); }
 
 if (read(fd,&wpcx,sizeof(pcx_header))!=sizeof(pcx_header))
  { perror(name); exit(1); }

 if ( (wpcx.xmax-wpcx.xmin)>319 || (wpcx.ymax-wpcx.ymin)>199)
  { fprintf(stderr,"%s: not a skin file\n",name); exit(1); }

 if (wpcx.planes!=1 || wpcx.bpp!=8)
  { fprintf(stderr,"%s: unsupported filetype\n",name); exit(1); }

 width=1;
 height=wpcx.xmax+1-wpcx.bytes_per_line;

 size=(long)(wpcx.xmax-wpcx.xmin+1)*(long)(wpcx.ymax-wpcx.ymin+1);
 data=(char *) malloc(size);
 x=0;
 while (x<size)
 {
  if (read(fd,r,1)!=1) { perror(name); exit(1); }
  if ( r[0]>191 )
   {
    if (read(fd,g,1)!=1) { perror(name); exit(1); }
    
    for (y=0; y< (r[0]&0x3f); y++)
     {
      data[x]=g[0];
      x++;
      
      if (width==wpcx.bytes_per_line && height!=0)
      {
       width=0;
       x--;
      }
      width++;
     }
   
   } else
   {
    data[x]=r[0];
    x++;
    if (width==wpcx.bytes_per_line && height!=0)
    {
     width=0;
     x--;
    }
    width++;
   }
 }
 
 if (lseek(fd,-768,SEEK_END)==-1) { perror(name); exit(1); }
 if (read(fd,palette,256*3)!=256*3  ) { perror(name); exit(1); }

 close(fd);

 return wpcx;
}
